/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.dialogs;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.eclipse.ui.model.WorkbenchLabelProvider;

public abstract class AbstractResourceSelectionDialog extends SelectionDialog {

	private TreeViewer treeViewer;
	private Tree tree;
	private final IResource[] resources;

	public AbstractResourceSelectionDialog(Shell parentShell, IResource... resources) {
		super(parentShell);
		this.resources = resources;
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	abstract protected boolean acceptSelection(IResource element);

	@Override
	protected Point getInitialSize() {
		return new Point(400, 400);
	}

	@Override
	public void create() {
		super.create();

		List<?> initialSelections = getInitialElementSelections();
		if (initialSelections.isEmpty()) {
			getOkButton().setEnabled(false);
		} else {
			final IResource resource = (IResource) initialSelections.get(0);
			if (resource != null) {
				updateOkButton(resource);

				for (final IResource res : resources) {
					try {
						res.accept(new IResourceVisitor() {
							public boolean visit(IResource c) throws CoreException {
								if (c != null && acceptSelection(c)) {
									if (c.getLocation().toOSString().equals(resource.getLocation().toOSString())) {

										// ツリーの子を指定しても選択状態とならないため親を指定
										//IStructuredSelection selection = new StructuredSelection(c);
										IStructuredSelection selection = new StructuredSelection(res);

										treeViewer.setSelection(selection, true);
										return false;
									}
								}
								return true;
							}
						});
					} catch (CoreException e) {
						throw new IllegalStateException(e);
					}
				}
			}
		}
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		createMessageArea(composite);

		treeViewer = new TreeViewer(composite, SWT.BORDER);
		tree = treeViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		treeViewer.setContentProvider(new TreeContentProvider());
		treeViewer.setLabelProvider(new WorkbenchLabelProvider());
		treeViewer.addFilter(new ViewerFilter() {
			@Override
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				if (element instanceof IProject) {
					IProject project = (IProject) element;
					if (!project.isAccessible()) {
						return false;
					}
				}
				return true;
			}
		});
		treeViewer.setInput(resources);
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection sselection = (IStructuredSelection) event.getSelection();
				IResource firstElement = (IResource) sselection.getFirstElement();
				updateOkButton(firstElement);
			}
		});
		return composite;
	}

	@Override
	protected void okPressed() {
		IStructuredSelection selection = (IStructuredSelection) treeViewer.getSelection();
		Object firstElement = selection.getFirstElement();
		List<Object> list = new ArrayList<Object>();
		list.add(firstElement);
		setResult(list);
		super.okPressed();
	}

	private void updateOkButton(IResource resource) {
		getOkButton().setEnabled(acceptSelection(resource));
	}

	//------------------------------------------------------------------------------------

	private class TreeContentProvider implements IStructuredContentProvider, ITreeContentProvider {

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof IProject[]) {
				IProject[] projects = (IProject[]) parentElement;
				return projects;
			}
			if (parentElement instanceof IContainer) {
				try {
					final IContainer container = (IContainer) parentElement;
					if (container.isAccessible()) {
						List<IResource> list = new LinkedList<IResource>();
						for (IResource member : container.members()) {
							if (member.getType() == IResource.FOLDER || acceptSelection(member)) {
								list.add(member);
							}
						}
						return list.toArray();
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return new Object[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}
	}
}
